@@ -1,37 +1,15 @@
+import os.path
+
 import gym
 import numpy as np
-
 from pydrake.all import (
-    AddMultibodyPlantSceneGraph,
-    Box,
-    ConstantVectorSource,
-    ContactVisualizer,
-    ContactVisualizerParams,
-    DiagramBuilder,
-    EventStatus,
-    FixedOffsetFrame,
-    InverseDynamicsController,
-    LeafSystem,
-    MeshcatVisualizerCpp,
-    MeshcatVisualizerParams,
-    MultibodyPlant,
-    MultibodyPositionToGeometryPose,
-    Multiplexer,
-    Parser,
-    PassThrough,
-    PlanarJoint,
-    PrismaticJoint,
-    RandomGenerator,
-    Rgba,
-    RigidTransform,
-    RotationMatrix,
-    SceneGraph,
-    Simulator,
-    SpatialInertia,
-    Sphere,
-    UnitInertia,
-    Variable,
-)
+    AddMultibodyPlantSceneGraph, Box, ConstantVectorSource, ContactVisualizer,
+    ContactVisualizerParams, DiagramBuilder, DiscreteContactSolver, EventStatus,
+    FixedOffsetFrame, InverseDynamicsController, LeafSystem, MeshcatVisualizer,
+    MeshcatVisualizerParams, MultibodyPlant, MultibodyPositionToGeometryPose,
+    Multiplexer, Parser, PassThrough, PlanarJoint, PrismaticJoint,
+    RandomGenerator, Rgba, RigidTransform, RotationMatrix, SceneGraph,
+    Simulator, SpatialInertia, Sphere, UnitInertia, Variable)
 
 from manipulation.drake_gym import DrakeGymEnv
 from manipulation.scenarios import AddShape, SetColor, SetTransparency
@@ -45,7 +23,8 @@ def AddPlanarBinAndSimpleBox(plant,
                              depth=0.05,
                              height=0.3):
     parser = Parser(plant)
-    bin = parser.AddModelFromFile(FindResource("models/planar_bin.sdf"))
+    sdf_location = FindResource("models/planar_bin.sdf")
+    bin = parser.AddModelFromFile(sdf_location)
     plant.WeldFrames(
         plant.world_frame(), plant.GetFrameByName("bin_base", bin),
         RigidTransform(RotationMatrix.MakeZRotation(np.pi / 2.0),
@@ -96,7 +75,13 @@ def make_box_flipup(generator,
                     time_limit=10):
     builder = DiagramBuilder()
     plant, scene_graph = AddMultibodyPlantSceneGraph(builder, time_step=0.001)
+
+    # TODO(russt): Re-enable Sap pending resolution of
+    # https://github.com/RobotLocomotion/drake/issues/18338
+    # plant.set_discrete_contact_solver(DiscreteContactSolver.kSap)
+
     # TODO(russt): randomize parameters.
+
     box = AddPlanarBinAndSimpleBox(plant)
     finger = AddPointFinger(plant)
     plant.Finalize()
@@ -106,7 +91,7 @@ def make_box_flipup(generator,
     AddPointFinger(controller_plant)
 
     if meshcat:
-        MeshcatVisualizerCpp.AddToBuilder(builder, scene_graph, meshcat)
+        MeshcatVisualizer.AddToBuilder(builder, scene_graph, meshcat)
         meshcat.Set2dRenderMode(xmin=-.35, xmax=.35, ymin=-0.1, ymax=0.3)
         ContactVisualizer.AddToBuilder(
             builder, plant, meshcat,
@@ -118,7 +103,7 @@ def make_box_flipup(generator,
         SetColor(controller_scene_graph,
                  color=[1.0, 165.0 / 255, 0.0, 1.0],
                  source_id=controller_plant.get_source_id())
-        controller_vis = MeshcatVisualizerCpp.AddToBuilder(
+        controller_vis = MeshcatVisualizer.AddToBuilder(
             builder, controller_scene_graph, meshcat,
             MeshcatVisualizerParams(prefix="controller"))
         controller_vis.set_name("controller meshcat")
@@ -219,19 +204,30 @@ def BoxFlipUpEnv(observations="state", meshcat=None, time_limit=10):
                                 observations,
                                 meshcat=meshcat,
                                 time_limit=time_limit)
-    action_space = gym.spaces.Box(low=np.array([-.5, -0.1], dtype="float32"),
-                                  high=np.array([.5, 0.6], dtype="float32"))
+    action_space = gym.spaces.Box(low=np.array([-.5, -0.1]),
+                                  high=np.array([.5, 0.6]),
+                                  dtype=np.float64)
 
     plant = simulator.get_system().GetSubsystemByName("plant")
 
+    # It is unsound to use raw MBP dof limits as observation bounds, because
+    # most simulations will violate those limits in practice (in collision, or
+    # due to gravity, or in general because no constraint force is incurred
+    # except in violation).  However we don't have any other better limits
+    # here.  So we broaden the limits by a fixed offset and hope for the best.
+    NUM_DOFS = 5
+    POSITION_LIMIT_TOLERANCE = np.full((NUM_DOFS,), 0.1)
+    VELOCITY_LIMIT_TOLERANCE = np.full((NUM_DOFS,), 0.5)
     if observations == "state":
         low = np.concatenate(
-            (plant.GetPositionLowerLimits(), plant.GetVelocityLowerLimits()))
+            (plant.GetPositionLowerLimits() - POSITION_LIMIT_TOLERANCE,
+             plant.GetVelocityLowerLimits() - VELOCITY_LIMIT_TOLERANCE))
         high = np.concatenate(
-            (plant.GetPositionUpperLimits(), plant.GetVelocityUpperLimits()))
-        observation_space = gym.spaces.Box(low=np.asarray(low, dtype="float32"),
-                                           high=np.asarray(high,
-                                                           dtype="float32"))
+            (plant.GetPositionUpperLimits() + POSITION_LIMIT_TOLERANCE,
+             plant.GetVelocityUpperLimits() + VELOCITY_LIMIT_TOLERANCE))
+        observation_space = gym.spaces.Box(low=np.asarray(low),
+                                           high=np.asarray(high),
+                                           dtype=np.float64)
 
     env = DrakeGymEnv(simulator=simulator,
                       time_step=0.1,
