@@ -230,8 +230,8 @@ class MeshcatPoseSliders(LeafSystem):
         return EventStatus.DidNothing()
 
     def Run(self, publishing_system, root_context, callback):
-        # Calls callback(root_context, pose), then publishing_system.Publish()
-        # each time the sliders change value.
+        # Calls callback(root_context, pose), then
+        # publishing_system.ForcedPublish() each time the sliders change value.
         if not running_as_notebook:
             return
 
@@ -239,11 +239,11 @@ class MeshcatPoseSliders(LeafSystem):
             root_context)
 
         print("Press the 'Stop PoseSliders' button in Meshcat to continue.")
-        self._meshcat.AddButton("Stop PoseSliders")
+        self._meshcat.AddButton("Stop PoseSliders", "Escape")
         while self._meshcat.GetButtonClicks("Stop PoseSliders") < 1:
             if self._update_values():
                 callback(root_context, self._get_transform())
-                publishing_system.Publish(publishing_context)
+                publishing_system.ForcedPublish(publishing_context)
             time.sleep(.1)
 
         self._meshcat.DeleteButton("Stop PoseSliders")
@@ -415,34 +415,26 @@ def plot_mathematical_program(meshcat,
                  result.get_optimal_cost()]))
 
 
-# TODO(russt): Use the one true Drake version once this lands:
-# https://github.com/RobotLocomotion/drake/issues/17689
-def model_inspector(meshcat, filename):
-    builder = DiagramBuilder()
+def PublishPositionTrajectory(trajectory,
+                              root_context,
+                              plant,
+                              visualizer,
+                              time_step=1.0 / 33.0):
+    """
+    Args:
+        trajectory: A Trajectory instance.
+    """
+    plant_context = plant.GetMyContextFromRoot(root_context)
+    visualizer_context = visualizer.GetMyContextFromRoot(root_context)
 
-    # Note: the time_step here is chosen arbitrarily.
-    plant, scene_graph = AddMultibodyPlantSceneGraph(builder, time_step=0.001)
+    visualizer.StartRecording(False)
 
-    # Load the file into the plant/scene_graph.
-    parser = Parser(plant)
-    parser.AddModelFromFile(filename)
-    plant.Finalize()
+    for t in np.append(
+            np.arange(trajectory.start_time(), trajectory.end_time(),
+                      time_step), trajectory.end_time()):
+        root_context.SetTime(t)
+        plant.SetPositions(plant_context, trajectory.value(t))
+        visualizer.ForcedPublish(visualizer_context)
 
-    # Add two visualizers, one to publish the "visual" geometry, and one to
-    # publish the "collision" geometry.
-    visual = MeshcatVisualizer.AddToBuilder(
-        builder, scene_graph, meshcat,
-        MeshcatVisualizerParams(role=Role.kPerception, prefix="visual"))
-    collision = MeshcatVisualizer.AddToBuilder(
-        builder, scene_graph, meshcat,
-        MeshcatVisualizerParams(role=Role.kProximity, prefix="collision"))
-    # Disable the collision geometry at the start; it can be enabled by the
-    # checkbox in the meshcat controls.
-    meshcat.SetProperty("collision", "visible", False)
-
-    # Set the timeout to a small number in test mode. Otherwise, JointSliders
-    # will run until "Stop JointSliders" button is clicked.
-    default_interactive_timeout = None if running_as_notebook else 1.0
-    sliders = builder.AddSystem(JointSliders(meshcat, plant))
-    diagram = builder.Build()
-    sliders.Run(diagram, default_interactive_timeout)
+    visualizer.StopRecording()
+    visualizer.PublishRecording()
