@@ -9,10 +9,10 @@ import warnings
 import numpy as np
 from pydrake.all import (
     AbstractValue, Adder, AddMultibodyPlantSceneGraph, BallRpyJoint, BaseField,
-    Box, CameraInfo, ClippingRange, CoulombFriction, Cylinder, Demultiplexer,
-    DepthImageToPointCloud, DepthRange, DepthRenderCamera, DiagramBuilder,
-    FindResourceOrThrow, GeometryInstance, InverseDynamicsController,
-    LeafSystem, LoadModelDirectives, LoadModelDirectivesFromString,
+    Box, CameraInfo, Capsule, ClippingRange, CoulombFriction, Cylinder,
+    Demultiplexer, DepthImageToPointCloud, DepthRange, DepthRenderCamera,
+    DiagramBuilder, FindResourceOrThrow, GeometryInstance,
+    InverseDynamicsController, LeafSystem, LoadModelDirectivesFromString,
     MakeMultibodyStateToWsgStateSystem, MakePhongIllustrationProperties,
     MakeRenderEngineVtk, ModelInstanceIndex, MultibodyPlant, Parser,
     PassThrough, PrismaticJoint, ProcessModelDirectives, RenderCameraCore,
@@ -92,11 +92,21 @@ def AddTwoLinkIiwa(plant, q0=[0.1, -1.2]):
 
 
 # TODO: take argument for whether we want the welded fingers version or not
-def AddWsg(plant, iiwa_model_instance, roll=np.pi / 2.0, welded=False):
+def AddWsg(plant,
+           iiwa_model_instance,
+           roll=np.pi / 2.0,
+           welded=False,
+           sphere=False):
     parser = Parser(plant)
     if welded:
-        gripper = parser.AddModelFromFile(
-            FindResource("models/schunk_wsg_50_welded_fingers.sdf"), "gripper")
+        if sphere:
+            gripper = parser.AddModelFromFile(
+                FindResource("models/schunk_wsg_50_welded_fingers_sphere.sdf"),
+                "gripper")
+        else:
+            gripper = parser.AddModelFromFile(
+                FindResource("models/schunk_wsg_50_welded_fingers.sdf"),
+                "gripper")
     else:
         gripper = parser.AddModelFromFile(
             FindResourceOrThrow(
@@ -109,26 +119,36 @@ def AddWsg(plant, iiwa_model_instance, roll=np.pi / 2.0, welded=False):
     return gripper
 
 
-def AddFloatingRpyJoint(plant, frame, instance):
+def AddFloatingXyzJoint(plant, frame, instance, actuators=False):
     inertia = UnitInertia.SolidSphere(1.0)
     x_body = plant.AddRigidBody(
         "x", instance,
         SpatialInertia(mass=0, p_PScm_E=[0., 0., 0.], G_SP_E=inertia))
-    plant.AddJoint(
+    x_joint = plant.AddJoint(
         PrismaticJoint("x", plant.world_frame(), x_body.body_frame(),
                        [1, 0, 0]))
+    if actuators:
+        plant.AddJointActuator("x", x_joint)
     y_body = plant.AddRigidBody(
         "y", instance,
         SpatialInertia(mass=0, p_PScm_E=[0., 0., 0.], G_SP_E=inertia))
-    plant.AddJoint(
+    y_joint = plant.AddJoint(
         PrismaticJoint("y", x_body.body_frame(), y_body.body_frame(),
                        [0, 1, 0]))
+    if actuators:
+        plant.AddJointActuator("y", y_joint)
+    z_joint = plant.AddJoint(
+        PrismaticJoint("z", y_body.body_frame(), frame, [0, 0, 1]))
+    if actuators:
+        plant.AddJointActuator("z", z_joint)
+
+
+def AddFloatingRpyJoint(plant, frame, instance):
+    inertia = UnitInertia.SolidSphere(1.0)
     z_body = plant.AddRigidBody(
         "z", instance,
         SpatialInertia(mass=0, p_PScm_E=[0., 0., 0.], G_SP_E=inertia))
-    plant.AddJoint(
-        PrismaticJoint("z", y_body.body_frame(), z_body.body_frame(),
-                       [0, 0, 1]))
+    AddFloatingXyzJoint(plant, z_body.body_frame(), instance)
     plant.AddJoint(BallRpyJoint("ball", z_body.body_frame(), frame))
 
 
@@ -143,8 +163,11 @@ def AddShape(plant, shape, name, mass=1, mu=1, color=[.5, .5, .9, 1.0]):
         inertia = UnitInertia.SolidCylinder(shape.radius(), shape.length())
     elif isinstance(shape, Sphere):
         inertia = UnitInertia.SolidSphere(shape.radius())
+    elif isinstance(shape, Capsule):
+        # TODO(russt): Add python bindings for SolidCapsule.
+        inertia = UnitInertia.SolidCylinder(shape.radius(), shape.length())
     else:
-        raise RunTimeError(
+        raise RuntimeError(
             f"need to write the unit inertia for shapes of type {shape}")
     body = plant.AddRigidBody(
         name, instance,
@@ -431,25 +454,35 @@ def AddIiwaDifferentialIK(builder, plant, frame=None):
     params = DifferentialInverseKinematicsParameters(plant.num_positions(),
                                                      plant.num_velocities())
     time_step = plant.time_step()
-    params.set_timestep(time_step)  # Not actually required.
     q0 = plant.GetPositions(plant.CreateDefaultContext())
     params.set_nominal_joint_position(q0)
+    params.set_end_effector_angular_speed_limit(2)
+    params.set_end_effector_translational_velocity_limits([-2, -2, -2],
+                                                          [2, 2, 2])
     if plant.num_positions() == 3:  # planar iiwa
         iiwa14_velocity_limits = np.array([1.4, 1.3, 2.3])
         params.set_joint_velocity_limits(
             (-iiwa14_velocity_limits, iiwa14_velocity_limits))
-        # These constants are in body frame.
-        params.set_end_effector_velocity_gain([.1, 0, 0, 0, .1, .1])
+        # These constants are in body frame
+        assert (
+            frame.name() == "iiwa_link_7"
+        ), "Still need to generalize the remaining planar diff IK params for different frames"  # noqa
+        params.set_end_effector_velocity_flag(
+            [True, False, False, True, False, True])
     else:
         iiwa14_velocity_limits = np.array([1.4, 1.4, 1.7, 1.3, 2.2, 2.3, 2.3])
         params.set_joint_velocity_limits(
             (-iiwa14_velocity_limits, iiwa14_velocity_limits))
-        params.set_end_effector_velocity_gain([.1] * 6)
+        params.set_joint_centering_gain(10 * np.eye(7))
     if frame is None:
         frame = plant.GetFrameByName("body")
     differential_ik = builder.AddSystem(
-        DifferentialInverseKinematicsIntegrator(plant, frame, time_step,
-                                                params))
+        DifferentialInverseKinematicsIntegrator(
+            plant,
+            frame,
+            time_step,
+            params,
+            log_only_when_result_state_changes=True))
     return differential_ik
 
 
@@ -459,6 +492,7 @@ def MakeManipulationStation(model_directives=None,
                             iiwa_prefix="iiwa",
                             wsg_prefix="wsg",
                             camera_prefix="camera",
+                            prefinalize_callback=None,
                             package_xmls=[]):
     """
     Creates a manipulation station system, which is a sub-diagram containing:
@@ -491,6 +525,10 @@ def MakeManipulationStation(model_directives=None,
         plant_setup_callback) starting with this prefix will get a camera
         attached.
 
+        prefinalize_callback: A function, setup(plant), that will be called
+        with the multibody plant before calling finalize.  This can be useful
+        for e.g. adding additional bodies/models to the simulation.
+
         package_xmls: A list of filenames to be passed to
         PackageMap.AddPackageXml().  This is useful if you need to add more
         models to your path (e.g. from your current working directory).
@@ -508,8 +546,9 @@ def MakeManipulationStation(model_directives=None,
         directives = LoadModelDirectivesFromString(model_directives)
         ProcessModelDirectives(directives, parser)
     if filename:
-        directives = LoadModelDirectives(filename)
-        ProcessModelDirectives(directives, parser)
+        parser.AddAllModelsFromFile(filename)
+    if prefinalize_callback:
+        prefinalize_callback(plant)
     plant.Finalize()
 
     for i in range(plant.num_model_instances()):
